<a class="sidebar-brand{% if logo %} centered{% endif %}" href="{{ pathto(master_doc) }}">
    {% block brand_content %}
    {%- if logo_url %}
    <div class="sidebar-logo-container">
    <img class="sidebar-logo" src="{{ logo_url }}" alt="Logo"/>
    </div>
    {%- endif %}
    {%- if theme_light_logo and theme_dark_logo %}
    <div class="sidebar-logo-container">
    <img class="sidebar-logo only-light" src="{{ pathto('_static/images/' + theme_light_logo, 1) }}" alt="Light Logo"/>
    <img class="sidebar-logo only-dark" src="{{ pathto('_static/images/' + theme_dark_logo, 1) }}" alt="Dark Logo"/>
    </div>
    {%- endif %}
    {#-
    {% if not theme_sidebar_hide_name %}
    <span class="sidebar-brand-text">{{ docstitle if docstitle else project }}</span>
    {%- endif %}
    -#}
    {% endblock brand_content %}
</a>

<div style="text-align: center;">
<!-- Place this tag in your head or just before your close body tag. -->
<script async defer src="https://buttons.github.io/buttons.js"></script>
<!--<iframe src="https://ghbtns.com/github-btn.html?user=moonshinelabs-ai&repo=moonshine&type=star&count=true" frameborder="0" scrolling="0" width="100" height="20" title="GitHub"></iframe>-->
<!-- Place this tag where you want the button to render. -->
<a class="github-button" href="https://github.com/moonshinelabs-ai/moonshine" data-color-scheme="no-preference: light; light: light; dark: dark;" data-size="large" data-show-count="true" aria-label="Star moonshinelabs-ai/moonshine on GitHub">Star</a>
</div>